#include "../../../lib/std.mi"
#include "attribs.m"
#include "utility.m"

Function reset();
Function createNotifier();
Function showNotifier(Int w);
Function onNext();
Function initStyle();
Function syncStyle();

Function Int fillNextTrackInfo(String corneroverride);
Function Int fillCustomInfo(String customstring);

Function checkPref(int bypassfs);

Global Container containerNotifier;
Global Layout layoutNotifier;
Global Timer timerNotifier;
Global String strLastAutoTitle;
Global Int intScreenMargin;
Global String strPosition;

Global String strNotifierStyleTargetsBackgrounds1;
Global String strNotifierStyleTargetsBackgrounds2;
Global String strNotifierStyleTargetsBackgrounds3;
Global String strNotifierStyleTargetsSoftButtons;

// ------------------------------------------------------------------------------
// init
// ------------------------------------------------------------------------------
System.onScriptLoaded() {
	initAttribs();
	initStyle();
	intScreenMargin = 10;
	timerNotifier = new Timer;
}

// ------------------------------------------------------------------------------
// shutdown
// ------------------------------------------------------------------------------
System.onScriptUnloading() {
	delete timerNotifier;
}

// ------------------------------------------------------------------------------
// called by the system when the global hotkey for notification is pressed
// ------------------------------------------------------------------------------
System.onShowNotification() {
	if (checkPref(1)) return;
	createNotifier();
	String str;
	if (getStatus() == STATUS_PLAYING) str = "Playing";
	if (getStatus() == STATUS_PAUSED) str = "Playback Paused";
	if (getStatus() == STATUS_STOPPED) str = "Playback Stopped";
	showNotifier(fillNextTrackInfo(str));
	complete; // prevents other scripts from getting the message
	return 1; // tells anybody else that might watch the returned value that, yes, we implemented that
}

// ------------------------------------------------------------------------------
// called by the system when the title for the playing item changes, this could be the result of the player
// going to the next track, or of an update in the track title
// ------------------------------------------------------------------------------
System.onTitleChange(String newtitle) {
	if (strLastAutoTitle == newtitle) return;
	if (StrLeft(newtitle, 1) == "[") {
		if (StrLeft(newtitle, 7) == "[Buffer" ||
				StrLeft(newtitle, 4) == "[ICY") return;
	}
	strLastAutoTitle = newtitle;
	onNext();
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the next button
// ------------------------------------------------------------------------------
onNext() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo(""));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onPlay() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo("Playing"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button
// ------------------------------------------------------------------------------
System.onPause() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillCustomInfo("Playback Paused"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button again
// ------------------------------------------------------------------------------
System.onResume() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo("Resuming Playback"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onStop() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillCustomInfo("End of Playback"));
}

// ------------------------------------------------------------------------------
// checks if we should display anything
// ------------------------------------------------------------------------------
Int checkPref(int bypassfs) {
	if (!bypassfs && configAttribute_notifierDisableFullscreen.getData() == "1" && isVideoFullscreen()) return 1;
	if (configAttribute_notifierNever.getData() == "1") return 1;
	if (configAttribute_notifierMinimized.getData() == "1" && !isMinimized()) return 1;
	if (configAttribute_notifierWindowshade.getData() == "1") {
		if (isMinimized()) return 0;
		Container c = getContainer("main");
		if (!c) return 1;
		Layout l = c.getCurLayout();
		if (!l) return 1;
		if (l.getId() != "shade") return 1;
	}
	return 0;
}

// ------------------------------------------------------------------------------
// fade in/out completed
// ------------------------------------------------------------------------------
layoutNotifier.onTargetReached() {
	if (strPosition == "IN") {
		timerNotifier.setDelay(StringToInteger(configAttribute_notifierHoldtime.getData()));
		timerNotifier.start();
	} else if (strPosition == "OUT") {
		reset();	
	}
}

// ------------------------------------------------------------------------------
// when notifier is clicked, bring back the app from minimized state if its minimized and focus it
// ------------------------------------------------------------------------------
layoutNotifier.onLeftButtonDown(int x, int y) {
	timerNotifier.stop();
	layoutNotifier.cancelTarget();
	reset();
	restoreApplication();
	activateApplication();
}

// ------------------------------------------------------------------------------
// close the notifier window, destroys the container automatically because it's dynamic
// ------------------------------------------------------------------------------
reset() {
	containerNotifier.close();
	containerNotifier = NULL;
	layoutNotifier = NULL;
}

// ------------------------------------------------------------------------------
createNotifier() {
	if (containerNotifier == NULL) {
		containerNotifier = newDynamicContainer("notifier");
		if (!containerNotifier) return; // reinstall duh!
			layoutNotifier = containerNotifier.getLayout("normal");
		if (!layoutNotifier) return; // reinstall twice, man
	} else {
		layoutNotifier.cancelTarget();
		timerNotifier.stop();
	}
}

// ------------------------------------------------------------------------------
showNotifier(int w) {
	syncStyle();
	
	int intW = w + 45;
	int intH = System.stringToInteger(layoutNotifier.getXMLParam("h"));

	int intRightOnScreenX = getViewportWidth() + getViewportLeft() - intW - intScreenMargin;
	int intRightOffScreenX = getViewportWidth() + getViewportLeft();
	int intLeftOnScreenX = intScreenMargin + getViewportLeft();
	int intLeftOffScreenX = (intW * -1) + getViewportLeft();
	
	int intTopOnScreenY = getViewportTop() + intScreenMargin;
	int intTopOffScreenY = (intH * -1) + getViewportTop() - 25;
	int intBottomOnScreenY = getViewportHeight() + getViewportTop() - intH - intScreenMargin;
	int intBottomOffScreenY = getViewportHeight() + getViewportTop() + 25; //accomodate taskbar height :(
	
	int intSelectedStartX;
	int intSelectedStartY;
	int intSelectedFinalX;
	int intSelectedFinalY;
	
	if (configAttribute_notifierFade.getData() == "1" || configAttribute_notifierNoAnimation.getData() == "1") {
		if (configAttribute_notifierPositionRight.getData() == "1") {
			if (configAttribute_notifierPositionBottom.getData() == "1") {
				intSelectedStartX = intRightOnScreenX;
				intSelectedFinalX = intRightOnScreenX;
				intSelectedStartY = intBottomOnScreenY;
				intSelectedFinalY = intBottomOnScreenY;
			} else {
				intSelectedStartX = intRightOnScreenX;
				intSelectedFinalX = intRightOnScreenX;
				intSelectedStartY = intTopOnScreenY;
				intSelectedFinalY = intTopOnScreenY;
			}
		} else {
			if (configAttribute_notifierPositionBottom.getData() == "1") {
				intSelectedStartX = intLeftOnScreenX;
				intSelectedFinalX = intLeftOnScreenX;
				intSelectedStartY = intBottomOnScreenY;
				intSelectedFinalY = intBottomOnScreenY;
			} else {
				intSelectedStartX = intLeftOnScreenX;
				intSelectedFinalX = intLeftOnScreenX;
				intSelectedStartY = intTopOnScreenY;
				intSelectedFinalY = intTopOnScreenY;
			}
		}
	} else {
		if (configAttribute_notifierPositionRight.getData() == "1") {
			if (configAttribute_notifierSlideVertical.getData() == "1") {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedStartX = intRightOnScreenX;
					intSelectedFinalX = intRightOnScreenX;
					intSelectedStartY = intBottomOffScreenY;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedStartX = intRightOnScreenX;
					intSelectedFinalX = intRightOnScreenX;
					intSelectedStartY = intTopOffScreenY;
					intSelectedFinalY = intTopOnScreenY;
				}
			} else {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedStartX = intRightOffScreenX;
					intSelectedFinalX = intRightOnScreenX;
					intSelectedStartY = intBottomOnScreenY;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedStartX = intRightOffScreenX;
					intSelectedFinalX = intRightOnScreenX;
					intSelectedStartY = intTopOnScreenY;
					intSelectedFinalY = intTopOnScreenY;
				}
			}
		} else {
			if (configAttribute_notifierSlideVertical.getData() == "1") {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedStartX = intLeftOnScreenX;
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedStartY = intBottomOffScreenY;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedStartX = intLeftOnScreenX;
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedStartY = intTopOffScreenY;
					intSelectedFinalY = intTopOnScreenY;
				}
			} else {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedStartX = intLeftOffScreenX;
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedStartY = intBottomOnScreenY;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedStartX = intLeftOffScreenX;
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedStartY = intTopOnScreenY;
					intSelectedFinalY = intTopOnScreenY;
				}
			}
		}
	}

	if (!layoutNotifier.isVisible()) {
		layoutNotifier.resize(intSelectedStartX, intSelectedStartY, intW, intH);
		if (configAttribute_notifierFade.getData() == "1") {
			layoutNotifier.setAlpha(0);
		} else {
			layoutNotifier.setAlpha(255);
		}
	}
	
	layoutNotifier.show();
	layoutNotifier.setTargetA(255);
	layoutNotifier.setTargetX(intSelectedFinalX);
	layoutNotifier.setTargetY(intSelectedFinalY);
	layoutNotifier.setTargetW(intW);
	layoutNotifier.setTargetH(intH);
	if (configAttribute_notifierNoAnimation.getData() == "1") {
		layoutNotifier.setTargetSpeed(0);
	} else {
		layoutNotifier.setTargetSpeed(StringToInteger(configAttribute_notifierSlideTime.getData()));
	}
	layoutNotifier.gotoTarget();
	strPosition = "IN";
}

// ------------------------------------------------------------------------------
// hold time elapsed
// ------------------------------------------------------------------------------
timerNotifier.onTimer() {
	stop();
	
	if (configAttribute_notifierFade.getData() == "1") {
		layoutNotifier.setTargetA(0);
		layoutNotifier.setTargetSpeed(StringToInteger(configAttribute_notifierSlideTime.getData()));
		layoutNotifier.gotoTarget();
	} else if(configAttribute_notifierNoAnimation.getData() == "1") {
		layoutNotifier.setTargetA(0);
		layoutNotifier.setTargetSpeed(0);
		layoutNotifier.gotoTarget();
	} else {
		int intW = w + 45;
		int intH = layoutNotifier.getHeight();
		
		int intRightOnScreenX = getViewportWidth() + getViewportLeft() - intW - intScreenMargin;
		int intRightOffScreenX = getViewportWidth() + getViewportLeft();
		int intLeftOnScreenX = intScreenMargin + getViewportLeft();
		int intLeftOffScreenX = (intW * -1) + getViewportLeft();
		
		int intTopOnScreenY = getViewportTop() + intScreenMargin;
		int intTopOffScreenY = (intH * -1) + getViewportTop() - 25;
		int intBottomOnScreenY = getViewportHeight() + getViewportTop() - intH - intScreenMargin;
		int intBottomOffScreenY = getViewportHeight() + getViewportTop() + 25; //accomodate taskbar height :(
		
		int intSelectedFinalX;
		int intSelectedFinalY;
		
		if (configAttribute_notifierPositionRight.getData() == "1") {
			if (configAttribute_notifierSlideVertical.getData() == "1") {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedFinalX = intRightOnScreenX;
					intSelectedFinalY = intBottomOffScreenY;
				} else {
					intSelectedFinalX = intRightOnScreenX;
					intSelectedFinalY = intTopOffScreenY;
				}
			} else {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedFinalX = intRightOffScreenX;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedFinalX = intRightOffScreenX;
					intSelectedFinalY = intTopOnScreenY;
				}
			}
		} else {
			if (configAttribute_notifierSlideVertical.getData() == "1") {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedFinalY = intBottomOffScreenY;
				} else {
					intSelectedFinalX = intLeftOnScreenX;
					intSelectedFinalY = intTopOffScreenY;
				}
			} else {
				if (configAttribute_notifierPositionBottom.getData() == "1") {
					intSelectedFinalX = intLeftOffScreenX;
					intSelectedFinalY = intBottomOnScreenY;
				} else {
					intSelectedFinalX = intLeftOffScreenX;
					intSelectedFinalY = intTopOnScreenY;
				}
			}
		}
		
		layoutNotifier.setTargetX(intSelectedFinalX);
		layoutNotifier.setTargetY(intSelectedFinalY);
		layoutNotifier.setTargetSpeed(StringToInteger(configAttribute_notifierSlideTime.getData()));
		layoutNotifier.gotoTarget();
	}
	strPosition = "OUT";
}

// ------------------------------------------------------------------------------
Int fillNextTrackInfo(String corneroverride) {
	Int maxv = 0;
	Int stream = 0;
	Group p = layoutNotifier;
	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	DebugString("got callback for " + getPlayItemString(), 0);
	if (StrLeft(getPlayItemString(), 7) == "http://") stream = 1;

	if (endofplayback) endofplayback.hide();

	if (plentry) { plentry.setText(integerToString(getPlaylistIndex()+1)+"/"+integerToString(getPlaylistLength())); plentry.show(); }
	if (nexttrack) {
		if (corneroverride == "") {
			if (!stream) {
				if (!isVideo())
					nexttrack.setText("New Track");
				else
					nexttrack.setText("New Video");
			}
			else nexttrack.setText("On Air");
		} else nexttrack.setText(corneroverride);
		nexttrack.show();
	}
	if (_title) {
		String str;
		if (!stream) {
			_title.setXmlParam("ticker", "0");
			_title.setXmlParam("display", "");
			str = getPlayitemMetaDataString("title"); 
			if (str == "") str = getPlayitemDisplayTitle();
			String l = getPlayItemMetaDataString("length");
			if (l != "") {
				str += " (" + integerToTime(stringtointeger(l)) + ")";
			}
			_title.setText(str);
		} else {
			_title.setXmlParam("ticker", "1");
			_title.setXmlParam("display", "songtitle");
			_title.setText("");
		}
		_title.show(); 
	}
	if (artist) { 
		if (!stream) {
			if (isVideo())
				artist.setText(""); 
			else
				artist.setText(getPlayitemMetaDataString("artist")); 
		} else {
			if (isVideo())
				artist.setText("Internet TV"); 
			else
				artist.setText("Internet Radio"); 
		}
		artist.show(); 
	}
	if (album) { 
		String str;
		if (!stream && !isVideo()) {
			album.setXmlParam("display", "");
			str = getPlayitemMetaDataString("album");
			String l = getPlayitemMetaDataString("track");
			if (l != "" && l != "-1") str += " (Track " + l + ")";
			album.setText(str); 
		} else {
			album.setText("");
			album.setXmlParam("display", "songinfo");
		}
		album.show(); 
	}
	maxv = artist.getAutoWidth();
	if (maxv < album.getAutoWidth()) maxv = album.getAutoWidth();
	if (maxv < _title.getAutoWidth()) maxv = _title.getAutoWidth();
	if (maxv < 128) maxv = 128;
	if (maxv > getViewportWidth()/4) maxv = getViewportWidth()/4;
	
	return maxv;
}

// ------------------------------------------------------------------------------
Int fillCustomInfo(String customtext) {
	Group p = layoutNotifier;
	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	if (plentry) { plentry.hide(); }
	if (nexttrack) nexttrack.hide();
	if (_title) { _title.hide(); }
	if (artist) { artist.hide(); }
	if (album) { album.hide(); }

	if (endofplayback) {
		endofplayback.setText(customtext);
		int aw = endofplayback.getAutoWidth();
		endofplayback.show();
		if (aw > 128)
			return aw;
	}
	return 128;
}

initStyle() {
	strNotifierStyleTargetsBackgrounds1 = "window.main.topleft;window.main.topmiddle;window.main.topright;window.main.middleleft;window.main.middleright;window.main.middleleft2;window.main.middleright2;window.main.bottomleft;window.main.bottommiddle;window.main.bottomright";
	strNotifierStyleTargetsBackgrounds2 = "window.main.footer.left;window.main.footer.middle;window.main.footer.right;window.main.shortfooter.left;window.main.shortfooter.middle;window.main.shortfooter.right;window.main.titlebar.left;window.main.titlebar.middle";
	strNotifierStyleTargetsBackgrounds3 = "window.main.titlebar.right;window.title.inset;window.title.pl;window.title.ml;window.title.vid;window.title.avs;window.title.notifier";
	strNotifierStyleTargetsSoftButtons = "window.button.system;window.button.switch;window.button.close";
	
	syncStyle();
}

syncStyle() {
	String strStyle;
	
	if (layoutNotifier) {
		if (configAttribute_styleCarbonWindows.getData() == "1") strStyle = "carbon"; else strStyle = "default";
		for (Int i = 0; i < (strCharCount(strNotifierStyleTargetsBackgrounds1, ";") + 1); i++) setStyle(layoutNotifier.findObject(System.getToken(strNotifierStyleTargetsBackgrounds1, ";", i)), strStyle);
		for (Int i = 0; i < (strCharCount(strNotifierStyleTargetsBackgrounds2, ";") + 1); i++) setStyle(layoutNotifier.findObject(System.getToken(strNotifierStyleTargetsBackgrounds2, ";", i)), strStyle);
		for (Int i = 0; i < (strCharCount(strNotifierStyleTargetsBackgrounds3, ";") + 1); i++) setStyle(layoutNotifier.findObject(System.getToken(strNotifierStyleTargetsBackgrounds3, ";", i)), strStyle);
		
		if (configAttribute_styleCarbonSoftButtons.getData() == "1") strStyle = "carbon"; else strStyle = "default";
		for (Int i = 0; i < (strCharCount(strNotifierStyleTargetsSoftButtons, ";") + 1); i++) setStyle(layoutNotifier.findObject(System.getToken(strNotifierStyleTargetsSoftButtons, ";", i)), strStyle);
	}
}

configAttribute_styleDefaultAll.onDataChanged() { syncStyle(); }
configAttribute_styleCarbonAll.onDataChanged() { syncStyle(); }
configAttribute_styleDefaultWindows.onDataChanged() { syncStyle(); }
configAttribute_styleCarbonWindows.onDataChanged() { syncStyle(); }
configAttribute_styleDefaultSoftButtons.onDataChanged() { syncStyle(); }
configAttribute_styleCarbonSoftButtons.onDataChanged() { syncStyle(); }